@@ -16,6 +16,7 @@ import functools
 import mock
 import netaddr
 import os
+import testtools
 
 from neutron.agent.common import config as agent_config
 from neutron.agent.common import ovs_lib
@@ -26,9 +27,9 @@ from neutron.agent.linux import external_process
 from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import utils as linux_utils
-from neutron.common import config as common_config
 from neutron.common import constants as l3_constants
 from neutron.common import utils as common_utils
+from neutron.conf import common as common_config
 from neutron.plugins.common import constants
 from neutron.services.provider_configuration import serviceprovider_opts
 from neutron.tests.common import l3_test_common
@@ -93,6 +94,48 @@ FAKE_IPSEC_CONNECTION = {
     "id": _uuid()
 }
 
+
+FAKE_IKE_POLICY_SHA256 = {
+    'auth_algorithm': 'sha256',
+    "ike_version": "v1",
+    'encryption_algorithm': 'aes-128',
+    'pfs': 'group5',
+    'phase1_negotiation_mode': 'main',
+    'lifetime_units': 'seconds',
+    'lifetime_value': 3600
+}
+
+FAKE_IPSEC_POLICY_SHA256 = {
+    "encapsulation_mode": "tunnel",
+    "encryption_algorithm": "aes-128",
+    "pfs": "group5",
+    "lifetime_units": "seconds",
+    "lifetime_value": 3600,
+    "transform_protocol": "esp",
+    "auth_algorithm": "sha256",
+}
+
+FAKE_IPSEC_CONNECTION_SHA256 = {
+    "vpnservice_id": _uuid(),
+    "status": "PENDING_CREATE",
+    "psk": "969022489",
+    "initiator": "bi-directional",
+    "admin_state_up": True,
+    "auth_mode": "psk",
+    'external_ip': "172.24.4.8",
+    "peer_cidrs": ["10.100.255.224/28"],
+    "mtu": 1500,
+    "dpd_action": "hold",
+    "dpd_interval": 30,
+    "dpd_timeout": 120,
+    "route_mode": "static",
+    "ikepolicy": FAKE_IKE_POLICY_SHA256,
+    "ipsecpolicy": FAKE_IPSEC_POLICY_SHA256,
+    "peer_address": "172.24.4.8",
+    "peer_id": "172.24.4.8",
+    "id": _uuid()
+}
+
 PUBLIC_NET = netaddr.IPNetwork('19.4.4.0/24')
 PRIVATE_NET = netaddr.IPNetwork('35.4.0.0/16')
 FAKE_PUBLIC_SUBNET_ID = _uuid()
@@ -184,8 +227,9 @@ class SiteInfo(object):
                                  'router_id': self.info['id'],
                                  'external_ip': str(self.public_net)})
 
-    def prepare_ipsec_conn_info(self, peer):
-        ipsec_connection = copy.deepcopy(FAKE_IPSEC_CONNECTION)
+    def prepare_ipsec_conn_info(self, peer, connection=FAKE_IPSEC_CONNECTION,
+                                local_id=None, peer_id=None):
+        ipsec_connection = copy.deepcopy(connection)
         local_cidrs = [str(s) for s in self.private_nets]
         peer_cidrs = [str(s) for s in peer.private_nets]
         ipsec_connection.update({
@@ -198,6 +242,10 @@ class SiteInfo(object):
             'local_cidrs': local_cidrs,
             'local_ip_vers': 4
         })
+        if local_id:
+            ipsec_connection['local_id'] = local_id
+        if peer_id:
+            ipsec_connection['peer_id'] = peer_id
         self.vpn_service['ipsec_site_connections'] = [ipsec_connection]
 
 
@@ -431,6 +479,20 @@ class TestIPSecBase(base.BaseSudoTestCase):
         site1.prepare_ipsec_conn_info(site2)
         site2.prepare_ipsec_conn_info(site1)
 
+    def prepare_ipsec_site_connections_sha256(self, site1, site2):
+        """Builds info for connections in both directions in prep for sync."""
+        site1.prepare_ipsec_conn_info(site2,
+                                    FAKE_IPSEC_CONNECTION_SHA256)
+        site2.prepare_ipsec_conn_info(site1,
+                                    FAKE_IPSEC_CONNECTION_SHA256)
+
+    def prepare_ipsec_site_connections_local_id(self, site1, site2):
+        """Builds info for connections in both directions in prep for sync."""
+        site1.prepare_ipsec_conn_info(site2, local_id='@site1.com',
+                                    peer_id='@site2.com')
+        site2.prepare_ipsec_conn_info(site1, local_id='@site2.com',
+                                    peer_id='@site1.com')
+
     def sync_to_create_ipsec_connections(self, site1, site2):
         """Perform a sync, so that connections are created."""
         # Provide service info to sync
@@ -507,6 +569,7 @@ class TestIPSecBase(base.BaseSudoTestCase):
 
 class TestIPSecScenario(TestIPSecBase):
 
+    @testtools.skip('bug/1598466')
     def test_single_ipsec_connection(self):
         site1 = self.create_site(PUBLIC_NET[4], [self.private_nets[1]])
         site2 = self.create_site(PUBLIC_NET[5], [self.private_nets[2]])
@@ -520,6 +583,35 @@ class TestIPSecScenario(TestIPSecBase):
         self.check_ping(site1, site2)
         self.check_ping(site2, site1)
 
+    @testtools.skip('bug/1598466')
+    def test_single_ipsec_connection_sha256(self):
+        site1 = self.create_site(PUBLIC_NET[4], [self.private_nets[1]])
+        site2 = self.create_site(PUBLIC_NET[5], [self.private_nets[2]])
+
+        self.check_ping(site1, site2, success=False)
+        self.check_ping(site2, site1, success=False)
+
+        self.prepare_ipsec_site_connections_sha256(site1, site2)
+        self.sync_to_create_ipsec_connections(site1, site2)
+
+        self.check_ping(site1, site2)
+        self.check_ping(site2, site1)
+
+    @testtools.skip('bug/1598466')
+    def test_single_ipsec_connection_local_id(self):
+        site1 = self.create_site(PUBLIC_NET[4], [self.private_nets[1]])
+        site2 = self.create_site(PUBLIC_NET[5], [self.private_nets[2]])
+
+        self.check_ping(site1, site2, success=False)
+        self.check_ping(site2, site1, success=False)
+
+        self.prepare_ipsec_site_connections_local_id(site1, site2)
+        self.sync_to_create_ipsec_connections(site1, site2)
+
+        self.check_ping(site1, site2)
+        self.check_ping(site2, site1)
+
+    @testtools.skip('bug/1598466')
     def test_ipsec_site_connections_with_mulitple_subnets(self):
         """Check with a pair of subnets on each end of connection."""
         site1 = self.create_site(PUBLIC_NET[4], self.private_nets[1:3])
@@ -537,6 +629,7 @@ class TestIPSecScenario(TestIPSecBase):
             self.check_ping(site1, site2, instance=i)
             self.check_ping(site2, site1, instance=i)
 
+    @testtools.skip('bug/1598466')
     def test_ipsec_site_connections_with_l3ha_routers(self):
         """Test ipsec site connection with HA routers.
         This test creates two agents. First agent will have Legacy and HA
@@ -580,6 +673,7 @@ class TestIPSecScenario(TestIPSecBase):
         self.check_ping(site1, site2, 0)
         self.check_ping(site2, site1, 0)
 
+    @testtools.skip('bug/1598466')
     def _test_admin_state_up(self, update_method):
         # Create ipsec connection between two sites
         site1 = self.create_site(PUBLIC_NET[4], [self.private_nets[1]])
@@ -612,11 +706,13 @@ class TestIPSecScenario(TestIPSecBase):
         self.check_ping(site1, site2)
         self.check_ping(site2, site1)
 
+    @testtools.skip('bug/1598466')
     def test_ipsec_site_connections_update_admin_state_up(self):
         """Test updating admin_state_up of ipsec site connections."""
 
         self._test_admin_state_up(self._update_ipsec_connection)
 
+    @testtools.skip('bug/1598466')
     def test_vpnservice_update_admin_state_up(self):
         """Test updating admin_state_up of a vpn service."""
 
